#### Setup Firebase Account

1. Create a Firebase Android project for the package name `at.asit.egovpushdemonstrator`
([instructions](https://firebase.google.com/docs/android/setup#manually_add_firebase)).
 
1. Download the `google-services.json` file ([instructions](https://support.google.com/firebase/answer/7015592)).

1. Copy the `google-services.json` file to [`android/app`](android/app) directory.

1. Generate and download a private key for the Firebase service account ([instructions](https://firebase.google.com/docs/cloud-messaging/auth-server#authorize_http_v1_send_requests)).

1. Rename the downloaded private key as `service-account.json` and copy it to [`server/resources/firebase`](server/resources/firebase)
directory.

1. Enable the Firebase Cloud Messaging API on the [Google API Console](https://console.developers.google.com/apis/library/fcm.googleapis.com)

#### Initialize ECDSA keys

Use the provided utility program to generate an ECDSA key pair:
```shell
$ ./server/tools/bin/ecdsa-key-pair-generator \
> --ecdsa_public_key_path=android/app/src/main/res/raw/sender_verification_key.dat \
> --ecdsa_private_key_path=server/resources/ecdsa/sender_signing_key.dat
```

#### Initialize TLS keys

Generate EC-based TLS keys:

1. Add the hostname of the server in which the demo application server will be run in the
`[test_sans]` section of the [`resources/tls/init_tls.cnf`](resources/tls/init_tls.cnf) file.
Note that this file already contains the hostnames that are typically used for local development
(`127.0.0.1` for `localhost`, and `10.0.2.2` for connecting to `localhost` from an emulated Android
device [more info](https://developer.android.com/studio/run/emulator-networking.html)).

1. Generate TLS keys:
```shell
$ cd server/resources/tls
$ openssl req -x509 -days 365 -nodes -newkey ec:<(openssl ecparam -name prime256v1) \
> -keyout tls_tmp.key -out tls.crt -config init_tls.cnf
$ openssl pkcs8 -topk8 -nocrypt -in tls_tmp.key -out tls.key
$ rm tls_tmp.key
$ cp tls.crt ../../../android/app/src/main/res/raw
```

## Adapt the server IP
To change the server IP, adapt the `host` and `hostPort` field in the following file:
`android/app/src/main/java/at/asit/egovpushdemonstrator/eRegistrationActivity.java`

## Run the Server

First, bundle the server into a tarball. Start in the main directory of the source code.
```shell
$ cd server && gradle build
```

At the end of the above process, you will see the tarball of the demo application server at 
`server/build/distributions/server.tar`. Next, copy that tarball to a
directory in the server in which the demo application server will be run. Finally, run the server:
```shell
$ tar -xf server.tar -C ../.. && cd ../..
$ ./server/bin/server \
> --port=8443 \
> --firebase_project_id=<FIREBASE_PROJECT_ID> \
> --service_account_credentials_path=resources/firebase/service-account.json \
> --ecdsa_private_key_path=resources/ecdsa/sender_signing_key.dat \
> --tls_cert_path=resources/tls/tls.crt \
> --tls_private_key_path=resources/tls/tls.key \
> --database_path=resources/sqlite/demo.db
```
