package at.asit.egovpushdemonstrator;

import android.os.AsyncTask;
import android.os.Build;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.View;
import android.widget.Switch;
import at.asit.egovpushdemonstrator.callables.DecryptSavedCiphertexts;
import at.asit.egovpushdemonstrator.callables.GenKey;
import at.asit.egovpushdemonstrator.callables.RegKey;
import at.asit.egovpushdemonstrator.common.KeyAlgorithm;
import com.google.capillary.android.KeyManager;
import io.grpc.ManagedChannel;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.security.GeneralSecurityException;
import java.util.concurrent.Callable;

public class ePrescriptionActivity extends AppCompatActivity {

    private ManagedChannel channel;
    private DemoCapillaryHandler handler;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_e_prescription);

        Utils.initialize(this);
        initChannelAndDependents();

        findViewById(R.id.btnCancel).setOnClickListener(v -> finish());
        findViewById(R.id.btnSave).setOnClickListener(v -> {
            try {
                saveSubscriptions();
                genAndRegKey(v);
            } catch (Exception e) {
                e.printStackTrace();
            }
            finish();
        });

        boolean fillingPrescription = eGovDataStore.getFillingPrescription(getApplicationContext());
        boolean intake = eGovDataStore.getIntake(getApplicationContext());
        boolean medicationList = eGovDataStore.getMedicationList(getApplicationContext());
        boolean newPrescription = eGovDataStore.getNewPrescription(getApplicationContext());

        ((Switch) findViewById(R.id.swFillingPrescription)).setChecked(fillingPrescription);
        ((Switch) findViewById(R.id.swIntake)).setChecked(intake);
        ((Switch) findViewById(R.id.swMedicationList)).setChecked(medicationList);
        ((Switch) findViewById(R.id.swNewPrescription)).setChecked(newPrescription);
    }

    private void saveSubscriptions() {
        boolean fillingPrescription = ((Switch) findViewById(R.id.swFillingPrescription)).isChecked();
        boolean intake = ((Switch) findViewById(R.id.swIntake)).isChecked();
        boolean medicationList = ((Switch) findViewById(R.id.swMedicationList)).isChecked();
        boolean newPrescription = ((Switch) findViewById(R.id.swNewPrescription)).isChecked();

        eGovDataStore.setFillingPrescription(getApplicationContext(), fillingPrescription);
        eGovDataStore.setIntake(getApplicationContext(), intake);
        eGovDataStore.setMedicationList(getApplicationContext(), medicationList);
        eGovDataStore.setNewPrescription(getApplicationContext(), newPrescription);
    }

    public void genAndRegKey(View view) throws Exception {
        KeyAlgorithm algorithm = KeyAlgorithm.RSA_ECDSA;
        KeyManager keyManager = Utils.getKeyManager(this, algorithm);
        new GenKey(keyManager, algorithm, true).call();
        new DemoAsyncTask(new RegKey(handler, keyManager, algorithm, true)).execute();
    }

    private void initChannelAndDependents() {
        try {
            channel = Utils.createGrpcChannel(this);
            handler = new DemoCapillaryHandler(this, channel);
            // For API levels 26 and above, decrypt any Capillary ciphertexts that have been saved since
            // the last launch of this activity. This is not needed if the ACTION_USER_PRESENT broadcast
            // receiver were to be watched in a background service.
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                KeyManager rsaEcdsaKeyManager = Utils.getKeyManager(this, KeyAlgorithm.RSA_ECDSA);
                new DemoAsyncTask(new DecryptSavedCiphertexts(handler,
                        rsaEcdsaKeyManager.getDecrypterManager(),
                        KeyAlgorithm.RSA_ECDSA)).execute();
                KeyManager webPushKeyManager = Utils.getKeyManager(this, KeyAlgorithm.WEB_PUSH);
                new DemoAsyncTask(new DecryptSavedCiphertexts(handler,
                        webPushKeyManager.getDecrypterManager(),
                        KeyAlgorithm.WEB_PUSH)).execute();
            }
        } catch (IOException | GeneralSecurityException e) {
            e.printStackTrace();
        }
    }

    private final class DemoAsyncTask extends AsyncTask<Void, Void, String> {

        private final Callable<String> callable;

        private DemoAsyncTask(Callable<String> callable) {
            this.callable = callable;
        }

        @Override
        protected String doInBackground(Void... nothing) {
            try {
                String logs = callable.call();
                return "Background task success!\n" + logs;
            } catch (Exception e) {
                e.printStackTrace();
                StringWriter sw = new StringWriter();
                PrintWriter pw = new PrintWriter(sw);
                e.printStackTrace(pw);
                pw.flush();
                return "Background task failure...\n" + sw;
            }
        }

        @Override
        protected void onPostExecute(String result) {
        }
    }

}
